--[[
	PickingSkill: A database of lockpicking data covering all looted, fished & stolen boxes.
	by Rhodri of Proudmoore

	Some methods and procedures derived, learnt &/or taken Gazzmik Fizzwidgets mods, ColourBlind, LinkWrangler & MailTo. 
--]]




-------------------------------
-- Constants & stored variables
-------------------------------
COLOUR_RED		= "|cFEFE1F1F"		-- Confirmed
COLOUR_ORANGE		= "|cFEFE7F3F"		-- Confirmed
COLOUR_GREEN		= "|cFE3FBF3F"		-- Confirmed
COLOUR_YELLOW		= "|cFEFEFE00"		-- Confirmed
COLOUR_GREY		= "|cDE7F7F7F"		-- Confirmed
COLOUR_WHITE		= "|cfefefefe"
PICKINGSKILL_VERSION	= "20000.1a"
local reqSkill
local currSkill
local SkillCap
local strDerivedColour

-------------------------------
-- Initialise the per character preference saved variables with default values
-------------------------------
PS_Enabled = true			
PS_Show_Key = true			
PS_Show_Skill = true
PS_Compact = false

-------------------------------
-- Hook Functions
-------------------------------

function PickingSkill_OnLoad()
	this:RegisterEvent("SKILL_LINES_CHANGED");
	this:RegisterForDrag("LeftButton");
--	this:RegisterEvent("ADDON_LOADED");

	SLASH_PICKINGSKILL1 = "/pickingskill";
	SLASH_PICKINGSKILL2 = "/ps"; 
	SlashCmdList["PICKINGSKILL"] = PickingSkill_CmdHandler;

	Original_Chat_OnHyperlinkShow = ChatFrame_OnHyperlinkShow;
	ChatFrame_OnHyperlinkShow = PickingSkill_Chat_OnHyperlinkShow;

	Original_ContainerFrameItemButton_OnEnter = ContainerFrameItemButton_OnEnter;
	ContainerFrameItemButton_OnEnter = PickingSkill_ContainerFrameItemButton_OnEnter;

	Original_GameTooltip_SetLootItem = GameTooltip.SetLootItem;
	GameTooltip.SetLootItem = PickingSkill_GameTooltip_SetLootItem;

	Original_InboxFrameItem_OnEnter = InboxFrameItem_OnEnter;
	InboxFrameItem_OnEnter = PickingSkill_InboxFrameItem_OnEnter;

--	Original_TradeFrameItemButton_OnEnter = TradeFrameItemButton_OnEnter;
--	TradeFrameItemButton_OnEnter = PickingSkill_TradeFrameItemButton_OnEnter;

	if (UnitClass("player") == PLAYER_CLASS) then
		local tempSkill = PickingSkill_GetSkill();
		if tempSkill ~= nil then
			currSkill = tempSkill
		end
	end

	-- Tooltip idea & method stolen from Archud
	this.name = this:GetName();
	getglobal(this.name.."_chkEnabled").tooltipText = PS_TEXT.CHKBOX.ENABLED;
	getglobal(this.name.."_chkShowKey").tooltipText = PS_TEXT.CHKBOX.SHOWKEY;	
	getglobal(this.name.."_chkShowSkill").tooltipText = PS_TEXT.CHKBOX.SHOWSKILL;
	getglobal(this.name.."_chkCompact").tooltipText = PS_TEXT.CHKBOX.COMPACT;


	local strLoaded = COLOUR_YELLOW..format(PS_TEXT.LOAD,PICKINGSKILL_VERSION)
	PS_Chat(strLoaded)
end


function PickingSkill_CmdHandler(cmd)
	local strConcat
	local strHelp
	cmd = string.lower(cmd)
	strConcat = COLOUR_YELLOW..format(PS_TEXT.TITLE,PICKINGSKILL_VERSION)
	local strHelp1 = format(PS_TEXT.STRINGS.HELP.LINE1,COLOUR_YELLOW,PS_TEXT.CMD.SKELETONKEY,COLOUR_WHITE)
	local strHelp2 = format(PS_TEXT.STRINGS.HELP.LINE2,COLOUR_YELLOW,PS_TEXT.CMD.SKILL,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE)
	local strHelp3 = format(PS_TEXT.STRINGS.HELP.LINE3,COLOUR_YELLOW,PS_TEXT.CMD.SHOW,COLOUR_WHITE)
	local strHelp4 = format(PS_TEXT.STRINGS.HELP.LINE4,COLOUR_YELLOW,PS_TEXT.CMD.COMPACT,COLOUR_WHITE)
	local strHelp5 = format(PS_TEXT.STRINGS.HELP.LINE5,COLOUR_YELLOW,PS_TEXT.CMD.OPTIONS,COLOUR_WHITE)
	strHelp = COLOUR_WHITE..strHelp1..strHelp2..strHelp3..strHelp4..strHelp5

	if (cmd == "") then
		strConcat = strConcat..strHelp

	elseif (cmd == PS_TEXT.CMD.COMPACT) then
		PS_Compact = not PS_Compact
		if (PS_Compact) then
			strConcat = strConcat.."\n"..COLOUR_WHITE..format(PS_TEXT.SHOWCOMPACT.YES,COLOUR_WHITE,COLOUR_RED,COLOUR_WHITE,COLOUR_GREY)
		else
			strConcat = strConcat.."\n"..COLOUR_WHITE..format(PS_TEXT.SHOWCOMPACT.NO,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE,COLOUR_GREY)
		end


	elseif (cmd == PS_TEXT.CMD.SKELETONKEY) then
		PS_Show_Key = not PS_Show_Key
		if (PS_Show_Key) then
			strConcat = strConcat.."\n"..COLOUR_WHITE..PS_TEXT.SHOWKEY.YES
		else
			strConcat = strConcat.."\n"..COLOUR_WHITE..PS_TEXT.SHOWKEY.NO
		end

	elseif (cmd == PS_TEXT.CMD.SKILL) then
		PS_Show_Skill = not PS_Show_Skill
		if (PS_Show_Skill) then
			strConcat = strConcat.."\n"..COLOUR_WHITE..format(PS_TEXT.SHOWSKILL.YES,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE,COLOUR_GREY)
		else
			strConcat = strConcat.."\n"..COLOUR_WHITE..format(PS_TEXT.SHOWSKILL.NO,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE,COLOUR_GREY)
		end

	elseif (cmd == PS_TEXT.CMD.SHOW) then
		strConcat = COLOUR_YELLOW..format(PS_TEXT.SHOWTITLE,PICKINGSKILL_VERSION)
		if (PS_Show_Skill) then
			strConcat = strConcat.."\n"..COLOUR_GREY..format(PS_TEXT.SHOWSKILL.YES,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE,COLOUR_GREY)
		else
			strConcat = strConcat.."\n"..COLOUR_GREY..format(PS_TEXT.SHOWSKILL.NO,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE,COLOUR_GREY)
		end
		if (PS_Show_Key) then
			strConcat = strConcat.."\n"..COLOUR_GREY..PS_TEXT.SHOWKEY.YES
		else
			strConcat = strConcat.."\n"..COLOUR_GREY..PS_TEXT.SHOWKEY.NO
		end
		if (PS_Compact) then
			strConcat = strConcat.."\n"..COLOUR_GREY..format(PS_TEXT.SHOWCOMPACT.YES,COLOUR_WHITE,COLOUR_RED,COLOUR_WHITE,COLOUR_GREY)
		else
			strConcat = strConcat.."\n"..COLOUR_GREY..format(PS_TEXT.SHOWCOMPACT.NO,COLOUR_WHITE,COLOUR_ORANGE,COLOUR_WHITE,COLOUR_GREY)
		end

	elseif (cmd == PS_TEXT.CMD.OPTIONS or cmd == PS_TEXT.CMD.CONFIG) then
		strConcat = strConcat..COLOUR_WHITE.."\n  Showing configuration window."
		PickingSkill_Main:Show();
	else
		strConcat = strConcat..COLOUR_WHITE.."\nCommand '"..COLOUR_GREY..cmd..COLOUR_WHITE.."' not recognised."..strHelp
	end

	if (strConcat) then
		PS_Chat(strConcat)
	end
end


function PickingSkill_OnEvent()
	if ( event == "SKILL_LINES_CHANGED" ) then
		local tempSkill = PickingSkill_GetSkill()
		if tempSkill ~= nil then
			currSkill = tempSkill
		end
	end
end


-- From ColourBlind
function PickingSkill_Chat_OnHyperlinkShow(link, text, button)
	-- Item was in chat channel
	Original_Chat_OnHyperlinkShow(link, text, button);
	
	if (IsAddOnLoaded("LinkWrangler")) then
		if (IR_MOD_AUCTIONEER_INDEX >0) then
			local frame = IR_MOD_ITEMREF_WINDOWS[IR_MOD_AUCTIONEER_INDEX].window;
			local link = IR_MOD_ITEMREF_WINDOWS[IR_MOD_AUCTIONEER_INDEX].itemRefID;
			itemID = PickingSkill_GetItemID(link)
			if itemID ~= nil then 
				reqSkill = getItem(itemID);
			end
			if reqSkill ~= nil then
				PickingSkill_ForceColour(reqSkill);
				sTooltip = setTooltipText(reqSkill)
				frame:AddLine(sTooltip,_,_,_,1);
				frame:Show();
			end
		end
	else
		if ( strsub(link, 1, 4) == "item" ) then
			itemID = PickingSkill_GetItemID(link)
			if itemID ~= nil then 
				reqSkill = getItem(itemID);
			end
			if reqSkill ~= nil then
				PickingSkill_ForceColour(reqSkill);
				sTooltip = setTooltipText(reqSkill)
				ItemRefTooltip:AddLine(sTooltip,_,_,_,1);
				ItemRefTooltip:Show();
			end
		end
	end
	reqSkill = nil	
end



-- From ColourBlind
function PickingSkill_ContainerFrameItemButton_OnEnter()
	-- Mouseovering items in backpack
	Original_ContainerFrameItemButton_OnEnter();

	local frameID = this:GetParent():GetID();
	local buttonID = this:GetID();
	local texture, itemCount, locked, quality, readable = GetContainerItemInfo(frameID, buttonID);
	local packitem = getItemIDofContainerItem()
	reqSkill = nil
	reqSkill = getItem(packitem)
	
	if reqSkill ~= nil then
		if PickingSkill_IsLocked(GameTooltip) then
			sTooltip = setTooltipText(reqSkill)
			GameTooltip:AddLine(sTooltip,_,_,_,1);
			GameTooltip:Show();
		end
	end
end


-- From ColourBlind
function PickingSkill_GameTooltip_SetLootItem(this, slot)
	Original_GameTooltip_SetLootItem(this, slot);
	local texture, lootitem, quantity, quality = GetLootSlotInfo(slot);
	-- GetLootSlotLink(slot) doesn't seem to work unfortunately.
	reqSkill = getItemFromName(lootitem)

	if reqSkill ~= nil then
		PickingSkill_ForceColour(reqSkill);
		sTooltip = setTooltipText(reqSkill)
		GameTooltip:AddLine(sTooltip,_,_,_,1);
		GameTooltip:Show();
	end
end


function PickingSkill_InboxFrameItem_OnEnter()
	-- Mouseovering items in the mailbox.
	Original_InboxFrameItem_OnEnter();
	local item = GetInboxHeaderInfo(this.index);
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	if (item) then
		local MailItemName = GetInboxItem(this.index);
		reqSkill = getItemFromName(MailItemName)
		if reqSkill ~= nil then
			PickingSkill_ForceColour(reqSkill);
			sTooltip = setTooltipText(reqSkill)
			GameTooltip:SetText(MailItemName);
			GameTooltip:AddLine(sTooltip,_,_,_,1);
			GameTooltip:Show();
		end		
	end
end


function PickingSkill_TradeFrameItemButton_OnEnter()
	-- Mouseovering items in trade window
--	Original_TradeFrameItemButton_OnEnter();
--[[	PS_Chat("TradeFrameOnEnter")

	local tradeitem = getItemIDofTradeItem()
	PS_Chat("TradeItemID:"..tradeitem)
	reqSkill = nil
	reqSkill = getItem(tradeitem)
	
	if reqSkill ~= nil then
		PS_Chat("reqSkill:"..reqSkill)
		if PickingSkill_IsLocked(GameTooltip) then
			sTooltip = setTooltipText(reqSkill)
			GameTooltip:AddLine(sTooltip,_,_,_,1);
			GameTooltip:Show();
			PS_Chat("Tooltip:"..stooltip)
		end
	end
]]
end



-------------------------------
-- Functions
-------------------------------

function PS_Chat(msg)
	if( DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(msg,1,1,1);
	end
end

function setTooltipText(reqSkill)
	local tempText
	local tempSkillSign

	if (UnitClass("player") == PS_TEXT.PLAYER_CLASS) then
		local tempSkill = PickingSkill_GetSkill();
		if tempSkill ~= nil then
			currSkill = tempSkill
		end

		if PS_Compact then			
			tempText = COLOUR_WHITE..format(PS_TEXT.STRINGS.COMPACT,strDerivedColour,reqSkill,COLOUR_WHITE)
		else
			if tonumber(reqSkill) <= tonumber(currSkill) then
				tempText = format(PS_TEXT.STRINGS.CAN,strDerivedColour,COLOUR_WHITE)
			else
				tempText = format(PS_TEXT.STRINGS.NOT,strDerivedColour,COLOUR_WHITE)
			end 
		end
		if PS_Show_Skill then
			if tonumber(reqSkill) < tonumber(currSkill) then
				tempSkillSign = "<"
			elseif tonumber(reqSkill) == tonumber(currSkill) then
				tempSkillSign = "="
			else
				tempSkillSign = ">"
			end
			tempText = tempText..COLOUR_WHITE.."\n("..strDerivedColour..reqSkill..COLOUR_WHITE..tempSkillSign..currSkill.."/"..SkillCap..")"
		end
	else
		if tonumber(reqSkill) <= 75 then 
			strSkeletonKey = PS_TEXT.STRINGS.SKELETON.SILVER
		elseif tonumber(reqSkill) <= 125 then
			strSkeletonKey = PS_TEXT.STRINGS.SKELETON.GOLD
		elseif tonumber(reqSkill) <= 175 then
			strSkeletonKey = PS_TEXT.STRINGS.SKELETON.TRUESILVER
		else
			strSkeletonKey = PS_TEXT.STRINGS.SKELETON.ARCANITE
		end		
		tempText = format(PS_TEXT.STRINGS.NOTROGUE,strDerivedColour,reqSkill,COLOUR_WHITE)
		if PS_Show_Key then
			tempText = tempText..format(PS_TEXT.STRINGS.KEYSTRING,strSkeletonKey)
		end
	end
	return tempText;	
end


function getItem(itemID)
	-- A returned value of nil indicates that either the item isn't pickable, or that it's
	-- just not something that is stored in the current database.
	-- Eg if a new 350 skill required fishing box is added to the game, the database will have
	-- to be rebuilt to add that item.
	-- It is possible to dynamically add new boxes to the database, I just haven't bothered doing it yet.

	if (PickingSkill_BoxInfo[itemID]) then
		local SkillRequired = PickingSkill_BoxInfo[itemID];
		return SkillRequired;
	end 

end


function getItemFromName(strItemName)
	-- A returned value of nil indicates that either the item isn't pickable, or that it's
	-- just not something that is stored in the current database.
	-- Eg if a new 350 skill required fishing box is added to the game, the database will have
	-- to be rebuilt to add that item.
	-- It is possible to dynamically add new boxes to the database, I just haven't bothered doing it yet.

	if (PickingSkill_BoxName[strItemName]) then
		local SkillRequired = PickingSkill_BoxName[strItemName];
		return SkillRequired;
	end 

end


-- From LinkWrangler
function PickingSkill_GetItemID(link)
	if link then
		return string.sub(link,6,string.find(link,":",6)-1);
	end 
end


function getItemIDofContainerItem()
	local g = GetMouseFocus(); 
	if g and g.hasItem then 
		local bag = g:GetParent():GetID(); 
		local slot = g:GetID(); 
		if not (bag == 0 and slot > 16) then 
			local link = GetContainerItemLink(bag, slot); 
			if (link) then 
				local _, _, itemID, id2, id3, id4 = string.find(link, "item%:(%d+)%:(%d+)%:(%d+)%:(%d+)"); 
				return itemID;
			else
				itemID = nil
				return itemID;
			end			
		end
	end
end


function getItemIDofTradeItem()
--[[	if ( this.hasItem ) then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		GameTooltip:SetTradePlayerItem(this:GetParent():GetID());
	end
]]
	
	local g = GetMouseFocus(); 
	if g and g.hasItem then 
		local slot = g:GetID(); 
		if (slot) then
			PS_Chat("TradeSlot:"..slot)
			local link = GetTradeTargetItemLink(slot)
			if (link) then 
				PS_Chat("Link:"..link)
				local _, _, itemID, id2, id3, id4 = string.find(link, "item%:(%d+)%:(%d+)%:(%d+)%:(%d+)"); 
				return itemID;
			else
				itemID = nil
				return itemID;
			end			
		end
	end
end


function PickingSkill_GetSkill()
	local nameSkill
	local tempSkill
	if (UnitClass("player") == PS_TEXT.PLAYER_CLASS) then
		SkillCap=UnitLevel("player")*5
		for i=1,GetNumSkillLines() do
			nameSkill, _, _, tempSkill = GetSkillLineInfo(i);
			if (nameSkill == PS_TEXT.SKILL_NAME) then
				return tempSkill
			end
		end
	end
end

function PickingSkill_IsLocked(frameTooltip)
local tempData
local tempLineText

	if (frameTooltip) then
		for i = 1, frameTooltip:NumLines() do
			tempData = getglobal(frameTooltip:GetName().."TextLeft"..i)
			tempLineText = tempData:GetText()
			if string.find(tempLineText,LOCKED,1) then
				r,g,b,a = tempData:GetTextColor()
				strDerivedColour = string.format("|c%02X%02X%02X%02X",a*255,r*255,g*255,b*255)
				return 1
			end
		end
	end
end

function PickingSkill_ForceColour(intReqSkill)
-- Chat link & looted items don't display locked status, so must fake the colour decision based on archival data
-- and if a Rogue, players current skill.
	
	if (intReqSkill) then
		if (UnitClass("player") == PS_TEXT.PLAYER_CLASS) then
			local tempSkill = PickingSkill_GetSkill();
			if tempSkill ~= nil then
				currSkill = tempSkill
			end
			if tonumber(intReqSkill) < tonumber(currSkill) then
				strDerivedColour = COLOUR_GREY	
			elseif tonumber(intReqSkill) == tonumber(currSkill) then
				strDerivedColour = COLOUR_ORANGE	
			else
				strDerivedColour = COLOUR_RED	
			end 
	--	else
	--		strDerivedColour = COLOUR_RED	
		end
	end
	if not (strDerivedColour) then
		strDerivedColour = COLOUR_RED
	end
end


function PickingSkillConfig_OnClick()
	if (this:GetName() == "PickingSkill_Main_chkEnabled") then
		PS_Enabled = this:GetChecked(); -- set profile
		PickingSkill_UpdateAbled()
	elseif (this:GetName() == "PickingSkill_Main_chkShowSkill") then
		PS_Show_Skill = this:GetChecked();
	elseif (this:GetName() == "PickingSkill_Main_chkShowKey") then
		PS_Show_Key = this:GetChecked();
	elseif (this:GetName() == "PickingSkill_Main_chkCompact") then
		PS_Compact = this:GetChecked();
	end
 end

function PickingSkill_OnDragStart()
	PickingSkill_Main:StartMoving()
end

function PickingSkill_OnDragStop()
	PickingSkill_Main:StopMovingOrSizing()
end

function PickingSkill_OnShow()
	getglobal(this:GetName().."_chkEnabled"):SetChecked( PS_Enabled );
	getglobal(this:GetName().."_chkShowSkill"):SetChecked( PS_Show_Skill );
	getglobal(this:GetName().."_chkShowKey"):SetChecked( PS_Show_Key );
	getglobal(this:GetName().."_chkCompact"):SetChecked( PS_Compact );
	PickingSkill_UpdateAbled()
end

function PickingSkill_UpdateAbled()
	if PS_Enabled then
		PickingSkill_Main_chkShowSkill:Enable()
		PickingSkill_Main_chkShowKey:Enable()
		PickingSkill_Main_chkCompact:Enable()
	else
		PickingSkill_Main_chkShowSkill:Disable()
		PickingSkill_Main_chkShowKey:Disable()
		PickingSkill_Main_chkCompact:Disable()
	end
end